package cz.uhk.dod.data.db

import androidx.room.Dao
import androidx.room.Insert
import androidx.room.OnConflictStrategy.REPLACE
import androidx.room.Query
import androidx.room.Transaction
import kotlinx.coroutines.flow.Flow

@Dao
interface DODDao {

    @Insert(onConflict = REPLACE)
    suspend fun insertFieldsOfStudy(fieldsOfStudy: List<FieldOfStudyEntity>)

    @Query("Delete From FieldOfStudyEntity")
    suspend fun deleteAllFieldsOfStudy()

    @Query("Select * From FieldOfStudyEntity")
    fun fieldsOfStudy(): Flow<List<FieldOfStudyEntity>>

    @Insert(onConflict = REPLACE)
    suspend fun insertEvents(events: List<EventEntity>)

    @Query("Delete From EventEntity")
    suspend fun deleteAllEvents()

    @Insert(onConflict = REPLACE)
    suspend fun insertVideos(videos: List<VideoEntity>)

    @Query("Delete From VideoEntity")
    suspend fun deleteAllVideos()

    @Query("Select * From EventEntity")
    fun events(): Flow<List<EventEntity>>

    @Query("Select * From VideoEntity")
    fun videos(): Flow<List<VideoEntity>>

    @Transaction
    suspend fun replaceFieldsOfStudy(fieldsOfStudy: List<FieldOfStudyEntity>) {
        deleteAllFieldsOfStudy()
        insertFieldsOfStudy(fieldsOfStudy)
    }

    @Transaction
    suspend fun replaceEvents(events: List<EventEntity>) {
        deleteAllEvents()
        insertEvents(events)
    }

    @Transaction
    suspend fun replaceVideos(videos: List<VideoEntity>) {
        deleteAllVideos()
        insertVideos(videos)
    }
}