package cz.uhk.dod.data.db

import androidx.room.Database
import androidx.room.RoomDatabase
import androidx.room.TypeConverter
import androidx.room.TypeConverters
import cz.uhk.dod.data.api.response.DegreeType
import cz.uhk.dod.data.api.response.FormType
import cz.uhk.dod.data.api.response.UniversityEventType

@Database(
    version = DbVersion,
    entities = [
        FieldOfStudyEntity::class,
        VideoEntity::class,
        EventEntity::class,
    ]
)
@TypeConverters(Converters::class)
abstract class Database : RoomDatabase() {

    abstract fun dodDAO(): DODDao

    companion object {
        const val DbName = "DODDatabase.db"
    }
}

private const val DbVersion = 1


class Converters {

    @TypeConverter fun fromDegreeType(value: DegreeType) = value.name
    @TypeConverter fun toDegreeType(value: String) =
        enumValues<DegreeType>().find { it.name == value }!!

    @TypeConverter fun fromFormType(value: FormType) = value.name
    @TypeConverter fun toFormType(value: String) =
        enumValues<FormType>().find { it.name == value }!!

    @TypeConverter fun fromEventType(value: UniversityEventType) = value.name
    @TypeConverter fun toEventType(value: String) =
        enumValues<UniversityEventType>().find { it.name == value }!!
}