package cz.uhk.dod.data.db

import androidx.room.Entity
import androidx.room.Ignore
import androidx.room.PrimaryKey
import cz.uhk.dod.data.api.response.DegreeType
import cz.uhk.dod.data.api.response.FormType
import cz.uhk.dod.data.api.response.UniversityEventType
import java.text.SimpleDateFormat
import java.util.*

@Entity(
    primaryKeys = ["shortCut", "degreeType"]
)
data class FieldOfStudyEntity(

    val shortCut: String,
    val degreeType: DegreeType,

    val name: String,
    val formType: FormType,
    val studyPeriod: Int,
    val description: String,
    val candidateProfile: String,
    val graduateProfile: String,
    val eApplicationFormUrl: String,
    val studyPlanUrl: String,
    val furtherStudyPossibility: String?,
    val examWaiverOption: String,
    val admissionProcedure: String?,
    val estimatedNumberOfAdmissions: Int,
    val accreditationUntil: String,
    val urlDocumentsToDownload: String?,
) {

    @delegate:Ignore
    val accreditation: Calendar by lazy {
        val sdf = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss")
        Calendar.getInstance().apply {
            timeInMillis = sdf.parse(accreditationUntil)!!.time
        }
    }
}

@Entity
data class VideoEntity(

    @PrimaryKey(autoGenerate = false)
    val url: String,
    val name: String,
    val thumbnail: String?,
    val description: String?,
)

@Entity(
    primaryKeys = ["name", "fromTime", "toTime"]
)
data class EventEntity(

    val name: String,
    val type: UniversityEventType,
    val fromTime: String,
    val toTime: String,
    val location: String,
    val repetitionTimeInMinutes: Int?,
    val description: String,
) {

    @delegate:Ignore
    val times: Pair<Calendar, Calendar> by lazy {
        val sdf = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss")
        Calendar.getInstance().apply {
            timeInMillis = sdf.parse(fromTime)!!.time
        } to Calendar.getInstance().apply {
            timeInMillis = sdf.parse(toTime)!!.time
        }
    }
}