package cz.uhk.dod.data.repo

import cz.uhk.dod.data.api.DODApi
import cz.uhk.dod.data.api.response.FieldOfStudy
import cz.uhk.dod.data.api.response.FieldOfStudyInfo
import cz.uhk.dod.data.api.response.UniversityEvent
import cz.uhk.dod.data.api.response.VideoInfo
import cz.uhk.dod.data.db.DODDao
import io.ktor.client.call.*
import org.koin.core.component.KoinComponent
import org.koin.core.component.inject

//KoinComponent pro pouzivani injectu
class DODRepo : KoinComponent {

    private val api by inject<DODApi>()
    private val dao by inject<DODDao>()

    val fieldsOfStudy = dao.fieldsOfStudy()
    val events = dao.events()
    val videos = dao.videos()

    suspend fun fetchAndSaveFieldOfStudy() =
        api.fieldOfStudy()
            .body<List<FieldOfStudy>>()
            .also {
                dao.replaceFieldsOfStudy(
                    it.map(FieldOfStudy::items)
                        .flatten()
                        .map(FieldOfStudyInfo::toEntity)
                )
            }

    suspend fun fetchAndSaveEvents() =
        api.events()
            .body<List<UniversityEvent>>()
            .also {
                dao.replaceEvents(
                    it.map(UniversityEvent::toEntity)
                )
            }

    suspend fun fetchAndSaveVideos() =
        api.videos()
            .body<List<VideoInfo>>()
            .also {
                dao.replaceVideos(
                    it.map(VideoInfo::toEntity)
                )
            }
}