package cz.uhk.dod.di

import androidx.room.Room
import cz.uhk.dod.data.api.DODApi
import cz.uhk.dod.data.db.Database
import cz.uhk.dod.data.db.Database.Companion.DbName
import cz.uhk.dod.data.repo.DODRepo
import cz.uhk.dod.ui.DODViewModel
import org.koin.android.ext.koin.androidApplication
import org.koin.androidx.viewmodel.dsl.viewModel
import org.koin.core.module.Module
import org.koin.dsl.module

val dataModule = module {
    repo()
    api()
    db()
}

val uiModule = module {
    viewModels()
}

val appModules = listOf(dataModule, uiModule)

private fun Module.viewModels() {
    viewModel { DODViewModel(get()) }
}

private fun Module.repo() {
    factory { DODRepo() }
}

private fun Module.api() {
    single { DODApi() }
}

private fun Module.db() {
    single {
        Room.databaseBuilder(
            androidApplication(),
            Database::class.java,
            DbName,
        ).build()
    }
    single { get<Database>().dodDAO() }
}