package cz.uhk.dod.theme

import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.material.MaterialTheme
import androidx.compose.material.darkColors
import androidx.compose.material.icons.Icons
import androidx.compose.material.lightColors
import androidx.compose.runtime.Composable
import androidx.compose.runtime.CompositionLocalProvider
import androidx.compose.runtime.ReadOnlyComposable
import androidx.compose.runtime.compositionLocalOf
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp

private val DarkColorPalette = darkColors(

    primary = Fim,
    primaryVariant = White,
    onPrimary = White,

    secondary = UhkDarkGrey,
    secondaryVariant = Black,
    onSecondary = White,

    background = BackgroundDark,
    onBackground = White,
)

private val LightColorPalette = lightColors(

    primary = Fim,
    primaryVariant = Black,
    onPrimary = White,

    secondary = UhkLightGrey,
    secondaryVariant = White,
    onSecondary = White,

    background = BackgroundLight,
    onBackground = Black,
)

data class Spacing(
    val tiny: Dp = 4.dp,
    val small: Dp = 8.dp,
    val medium: Dp = 16.dp,
    val large: Dp = 24.dp,
    val double: Dp = 32.dp,
)

data class Elevation(
    val none: Dp = 0.dp,
    val small: Dp = 2.dp,
    val medium: Dp = 4.dp,
    val large: Dp = 8.dp,
)

private val LocalSpacing = compositionLocalOf { Spacing() }
private val LocalElevation = compositionLocalOf { Elevation() }

val MaterialTheme.spacing: Spacing
    @Composable
    @ReadOnlyComposable
    get() = LocalSpacing.current

val MaterialTheme.elevation: Spacing
    @Composable
    @ReadOnlyComposable
    get() = LocalSpacing.current

@Composable
fun FimTheme(
    darkTheme: Boolean = isSystemInDarkTheme(),
    content: @Composable () -> Unit,
) {
    CompositionLocalProvider(
        LocalSpacing provides Spacing(),
        LocalElevation provides Elevation(),
    ) {
        MaterialTheme(
            colors = if (darkTheme) DarkColorPalette
            else LightColorPalette,
            typography = Typography,
            content = content,
        )
    }
}

@Composable
fun Fim3Theme(
    content: @Composable () -> Unit,
) {
    androidx.compose.material3.MaterialTheme(
        typography = Typography3,
        content = content,
    )
}

typealias FimIcons = Icons.Outlined