package cz.uhk.dod.tools

import cz.uhk.dod.BuildConfig
import io.ktor.client.*
import io.ktor.client.plugins.*
import io.ktor.client.plugins.contentnegotiation.*
import io.ktor.client.plugins.logging.*
import io.ktor.client.statement.*
import io.ktor.serialization.kotlinx.json.*
import kotlinx.serialization.json.Json
import java.net.SocketTimeoutException
import java.net.UnknownHostException


fun HttpClientConfig<*>.installLogging(isDebug: Boolean = BuildConfig.DEBUG) {
    install(Logging) {
        logger = Logger.DEFAULT
        level = if (isDebug) LogLevel.ALL
        else LogLevel.NONE
    }
}

fun HttpClientConfig<*>.installJsonSerialization(isDebug: Boolean = BuildConfig.DEBUG) {
    install(ContentNegotiation) {
        json(
            Json {
                prettyPrint = true
                isLenient = true
                ignoreUnknownKeys = true
                explicitNulls = false
            }
        )
    }
}

fun HttpClientConfig<*>.installTimeout() {
    install(HttpTimeout) {
        requestTimeoutMillis = 5000L
        connectTimeoutMillis = 2500L
    }
}

val HttpResponse.isSuccessful
    get() = status.value in 200..299

val Exception.isNetworkConnection
    get() = when (this) {
        is SocketTimeoutException,
        is NullPointerException,
        is IllegalStateException,
        is UnknownHostException,
        -> true
        else
        -> false
    }