package cz.uhk.dod.ui

import cz.uhk.dod.data.api.response.UniversityEventType
import cz.uhk.dod.data.repo.DODRepo
import cz.uhk.dod.ui.base.ComposeViewModel
import cz.uhk.dod.ui.base.LaunchState
import cz.uhk.dod.ui.base.None
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.combine
import kotlinx.coroutines.flow.map

class DODViewModel(
    private val repo: DODRepo,
) : ComposeViewModel() {

    private val _infoState = MutableStateFlow<LaunchState>(None)
    val infoState = _infoState.asStateFlow()

    private val _eventState = MutableStateFlow<LaunchState>(None)
    val eventState = _eventState.asStateFlow()

    private val _videoState = MutableStateFlow<LaunchState>(None)
    val videoState = _videoState.asStateFlow()

    val fieldsOfStudy = repo.fieldsOfStudy.map { list ->
        list.groupBy { it.degreeType }
    }

    private val eventType = MutableStateFlow(UniversityEventType.PRESENTATION)
    val events = repo.events.combine(eventType) { events, type ->
        type to events.filter { item -> item.type == type }
    }

    val videos = repo.videos

    init {
        fetchAll()
    }

    fun fetchAll() {
        launch {
            repo.fetchAndSaveFieldOfStudy()
            repo.fetchAndSaveEvents()
            repo.fetchAndSaveVideos()
        }
    }

    fun fetchInfo() {
        launch(state = _infoState) {
            repo.fetchAndSaveFieldOfStudy()
        }
    }

    fun fetchEvents() {
        launch(state = _eventState) {
            repo.fetchAndSaveEvents()
        }
    }

    fun fetchVideos() {
        launch(state = _videoState) {
            repo.fetchAndSaveVideos()
        }
    }

    fun selectEventType(ordinalEventType: Int) {
        eventType.emit { enumValues<UniversityEventType>()[ordinalEventType] }
    }
}