@file:OptIn(ExperimentalMaterial3Api::class)

package cz.uhk.dod.ui.pages

import android.content.Intent
import android.net.Uri
import androidx.activity.ComponentActivity
import androidx.compose.animation.rememberSplineBasedDecay
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material.MaterialTheme
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.twotone.*
import androidx.compose.material3.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import cz.uhk.dod.theme.spacing
import cz.uhk.dod.ui.Routes

@Composable
fun ContactScreen() {

    val decayAnimationSpec = rememberSplineBasedDecay<Float>()
    val scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(
        decayAnimationSpec = decayAnimationSpec,
        state = rememberTopAppBarState()
    )

    Scaffold(
        modifier = Modifier.nestedScroll(scrollBehavior.nestedScrollConnection),
        topBar = {
            LargeTopAppBar(
                title = {
                    Text(
                        text = stringResource(Routes.Contacts.label)
                    )
                },
                colors = TopAppBarDefaults.largeTopAppBarColors(
                    containerColor = MaterialTheme.colors.background,
                    titleContentColor = MaterialTheme.colors.onBackground,
                ),
                scrollBehavior = scrollBehavior
            )
        },
        containerColor = MaterialTheme.colors.background,
        content = { innerPadding ->

            LazyColumn(
                contentPadding = PaddingValues(
                    top = innerPadding.calculateTopPadding(),
                    start = MaterialTheme.spacing.medium,
                    end = MaterialTheme.spacing.medium,
                    bottom = MaterialTheme.spacing.large,
                ),
                verticalArrangement = Arrangement.spacedBy(MaterialTheme.spacing.medium)
            ) {

                item {
                    Spacer(Modifier.height(MaterialTheme.spacing.medium))
                    IconText(
                        icon = Icons.TwoTone.LocationCity,
                        space = MaterialTheme.spacing.small,
                        subtext = {
                            val address = "Hradecká 1249/6\n500 03 Hradec Králové"
                            val activity = LocalContext.current as ComponentActivity
                            Text(
                                text = address,
                                style = MaterialTheme.typography.h5,
                                color = MaterialTheme.colors.onBackground.copy(alpha = 0.5F),
                                modifier = Modifier
                                    .clickable {
                                        val intent = Intent(Intent.ACTION_VIEW,
                                            Uri.parse("http://maps.google.co.in/maps?q=" + address))
                                        try {
                                            activity.startActivity(intent)
                                        } catch (_: Exception) {
                                        }
                                    }
                            )
                        }
                    ) {
                        Text(
                            text = "Sídlo fakulty",
                            style = MaterialTheme.typography.h4,
                            color = MaterialTheme.colors.onBackground,
                        )
                    }
                }

                item {
                    Spacer(Modifier.height(MaterialTheme.spacing.medium))
                    IconText(
                        icon = Icons.TwoTone.Phone,
                        space = MaterialTheme.spacing.small,
                        subtext = {
                            val phone = "+420 493 331 111"
                            val activity = LocalContext.current as ComponentActivity
                            Text(
                                text = phone,
                                style = MaterialTheme.typography.h5,
                                color = MaterialTheme.colors.onBackground.copy(alpha = 0.5F),
                                modifier = Modifier
                                    .clickable {
                                        val intent = Intent(
                                            Intent.ACTION_DIAL,
                                            Uri.fromParts("tel", phone, null)
                                        )
                                        try {
                                            activity.startActivity(intent)
                                        } catch (_: Exception) {
                                        }
                                    }
                            )
                        }
                    ) {
                        Text(
                            text = "Telefon",
                            style = MaterialTheme.typography.h4,
                            color = MaterialTheme.colors.onBackground,
                        )
                    }
                }

                item {
                    Spacer(Modifier.height(MaterialTheme.spacing.medium))
                    IconText(
                        icon = Icons.TwoTone.MeetingRoom,
                        space = MaterialTheme.spacing.small,
                        subtext = {
                            Text(
                                text = "pondělí - pátek: 6.00 - 21.30",
                                style = MaterialTheme.typography.h5,
                                color = MaterialTheme.colors.onBackground.copy(alpha = 0.5F),
                            )
                        }
                    ) {
                        Text(
                            text = "Provozní doba",
                            style = MaterialTheme.typography.h4,
                            color = MaterialTheme.colors.onBackground,
                        )
                    }
                }

                item {
                    Spacer(Modifier.height(MaterialTheme.spacing.medium))
                    IconText(
                        icon = Icons.TwoTone.ShareLocation,
                        space = MaterialTheme.spacing.small,
                        subtext = {
                            Text(
                                text = "Budova J se nachází v kampusu univerzity Na Soutoku " +
                                        "v centru Hradce Králové. Při dopravě automobilem " +
                                        "můžete využít parkoviště v areálu kampusu. Při cestě " +
                                        "městskou hromadnou dopravou vystupte na zastávce " +
                                        "Heyrovského (linky 1, 9, 23, 24 a 28) a areál se " +
                                        "bude nalézat přímo před Vámi. Případně zvolte spojení " +
                                        "na zastávku Zimní stadion (linky 2, 16, 18, 23 a 27), " +
                                        "pokračujte po mostě přes řeku Orlici podél Studijní a " +
                                        "vědecké knihovny v Hradci Králové, kampus se nalézá " +
                                        "za knihovnou na pravé straně.",
                                style = MaterialTheme.typography.h5,
                                color = MaterialTheme.colors.onBackground.copy(alpha = 0.5F),
                            )
                        }
                    ) {
                        Text(
                            text = "Umístění",
                            style = MaterialTheme.typography.h4,
                            color = MaterialTheme.colors.onBackground,
                        )
                    }
                }

                item {
                    Spacer(Modifier.height(MaterialTheme.spacing.medium))
                    IconText(
                        icon = Icons.TwoTone.Public,
                        space = MaterialTheme.spacing.small,
                        subtext = {
                            val url = "https://www.uhk.cz/cs/fakulta-informatiky-a-managementu"
                            val activity = LocalContext.current as ComponentActivity
                            Text(
                                text = url,
                                style = MaterialTheme.typography.h5,
                                color = MaterialTheme.colors.onBackground.copy(alpha = 0.5F),
                                modifier = Modifier
                                    .clickable {
                                        val intent = Intent(Intent.ACTION_VIEW, Uri.parse(url))
                                        try {
                                            activity.startActivity(intent)
                                        } catch (_: Exception) {
                                        }
                                    }
                            )
                        }
                    ) {
                        Text(
                            text = "Webové stránky",
                            style = MaterialTheme.typography.h4,
                            color = MaterialTheme.colors.onBackground,
                        )
                    }
                }
            }
        }
    )
}