@file:OptIn(ExperimentalMaterial3Api::class)

package cz.uhk.dod.ui.pages

import android.content.Intent
import android.net.Uri
import androidx.compose.animation.rememberSplineBasedDecay
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.MaterialTheme
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.twotone.Sync
import androidx.compose.material3.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import coil.compose.AsyncImage
import coil.request.ImageRequest
import cz.uhk.dod.R
import cz.uhk.dod.theme.spacing
import cz.uhk.dod.ui.DODViewModel
import cz.uhk.dod.ui.EmptyView
import cz.uhk.dod.ui.Routes

@Composable
fun MoreScreen(
    viewModel: DODViewModel,
) {

    val decayAnimationSpec = rememberSplineBasedDecay<Float>()
    val scrollBehavior = TopAppBarDefaults.exitUntilCollapsedScrollBehavior(
        decayAnimationSpec = decayAnimationSpec,
        state = rememberTopAppBarState()
    )

    val videos = viewModel.videos.collectAsState(emptyList())

    Scaffold(
        modifier = Modifier.nestedScroll(scrollBehavior.nestedScrollConnection),
        topBar = {
            LargeTopAppBar(
                title = {
                    Text(
                        text = stringResource(Routes.More.label)
                    )
                },
                colors = TopAppBarDefaults.largeTopAppBarColors(
                    containerColor = MaterialTheme.colors.background,
                    titleContentColor = MaterialTheme.colors.onBackground,
                ),
                actions = {
                    IconButton(
                        onClick = {
                            viewModel.fetchVideos()
                        }
                    ) {
                        Icon(
                            imageVector = Icons.TwoTone.Sync,
                            contentDescription = "Refresh",
                        )
                    }
                },
                scrollBehavior = scrollBehavior
            )
        },
        containerColor = MaterialTheme.colors.background,
        content = { innerPadding ->

            LazyColumn(
                contentPadding = PaddingValues(
                    top = innerPadding.calculateTopPadding(),
                    start = MaterialTheme.spacing.medium,
                    end = MaterialTheme.spacing.medium,
                    bottom = MaterialTheme.spacing.large,
                ),
                verticalArrangement = Arrangement.spacedBy(MaterialTheme.spacing.medium)
            ) {

                if (videos.value.isNotEmpty()) {
                    items(videos.value) { video ->
                        val activity = LocalContext.current
                        Row(
                            modifier = Modifier
                                .fillMaxWidth()
                                .clickable {
                                    val intent = Intent(Intent.ACTION_VIEW, Uri.parse(video.url))
                                    activity.startActivity(intent)
                                }
                                .padding(all = MaterialTheme.spacing.medium)
                        ) {
                            AsyncImage(
                                model = ImageRequest.Builder(LocalContext.current)
                                    .data(video.thumbnail)
                                    .crossfade(true)
                                    .error(R.drawable.ic_fim)
                                    .build(),
                                contentScale = ContentScale.Crop,
                                contentDescription = video.url,
                                modifier = Modifier
                                    .width(100.dp)
                                    .aspectRatio(1F),
                            )
                            Spacer(Modifier.width(MaterialTheme.spacing.medium))
                            Column(
                                modifier = Modifier.weight(1F)
                            ) {
                                Text(
                                    text = video.name,
                                    style = MaterialTheme.typography.h4,
                                )
                                if (video.description != null) {
                                    Text(
                                        text = video.description.orEmpty(),
                                        style = MaterialTheme.typography.body1,
                                    )
                                }
                            }
                        }
                    }
                } else item { EmptyView(viewModel::fetchVideos) }
            }
        }
    )
}